package com.huasheng.quant.open.gateway.sdk.domain;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * 获取调用结果前先判断是成功
 */
public class ModelResult<T> implements Serializable {
    private static final long serialVersionUID = 5652937879470313259L;

    private Map<String, String> errorList = new HashMap<>();

    private T model;

    public ModelResult() {
    }

    public ModelResult(T model) {
        this.model = model;
    }

    public T getModel() {
        return model;
    }

    public void setModel(T model) {
        this.model = model;
    }

    public ModelResult<T> withModel(T model) {
        this.model = model;
        return this;
    }

    public String getErrorCode() {
        if (errorList.isEmpty()) {
            return null;
        }
        return errorList.keySet().iterator().next();
    }

    public String getErrorMsg() {
        if (errorList.isEmpty()) {
            return "";
        }
        return errorList.values().iterator().next();
    }

    public ModelResult<T> withError(String errorCode, String errorMsg) {
        errorList.put(errorCode, errorMsg);
        return this;
    }

    /**
     * 判断是否成功
     */
    public boolean isSuccess() {
        return errorList.isEmpty();
    }

}