package com.huasheng.quant.open.gateway.sdk.handlers;

import com.huasheng.quant.open.gateway.api.HSQuantOpenApiHandle;
import com.huasheng.quant.open.gateway.sdk.notify.NotifyMessageRunnable;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufNofityMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufResponseMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.utils.future.SyncFutureUtil;
import io.netty.channel.ChannelHandlerContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class DefaultClientApplication extends ApplicationAdapter {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    protected NotifyMessageRunnable notifyMessageRunnable;
    protected HSQuantOpenApiHandle hsQuantOpenApiHandle;

    public DefaultClientApplication(NotifyMessageRunnable notifyMessageRunnable,
                                    HSQuantOpenApiHandle hsQuantOpenApiHandle) {
        this.notifyMessageRunnable = notifyMessageRunnable;
        this.hsQuantOpenApiHandle = hsQuantOpenApiHandle;
    }

    @Override
    public void onMessage(ChannelHandlerContext ctx, SocketMessage msg, List<Object> out) {
        if (msg instanceof ProtoBufResponseMessage) {
            SyncFutureUtil.receivedResult((ProtoBufResponseMessage) msg);
        }
        if (msg instanceof ProtoBufNofityMessage) {
            notifyMessageRunnable.push((ProtoBufNofityMessage) msg);
        }
//		logger.info("接收结果：" + msg.getHeader().toString() + " " + msg.getMsgBody().toString());
        super.onMessage(ctx, msg, out);
    }

    @Override
    public void beforeSendMessage(ChannelHandlerContext ctx, SocketMessage msg) {
//		logger.info("发送请求：" + msg.getHeader().toString() + " " + msg.getMsgBody().toString());
        super.beforeSendMessage(ctx, msg);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        logger.error("发生异常", cause);
        super.exceptionCaught(ctx, cause);
        if (this.hsQuantOpenApiHandle != null) {
            logger.info("Attempt to reconnect server.");
            this.hsQuantOpenApiHandle.reconnect();// 尝试重连
        }
    }

    public NotifyMessageRunnable getPushMessageNotifyHandle() {
        return notifyMessageRunnable;
    }

    public HSQuantOpenApiHandle getHsQuantOpenApiHandle() {
        return hsQuantOpenApiHandle;
    }
}
