package com.huasheng.quant.open.gateway.sdk.netty.codec;

import com.huasheng.quant.open.gateway.sdk.constant.ISystemError;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.RequestProto.PBRequest;
import com.huasheng.quant.open.gateway.sdk.protocol.MessageHeader;
import com.huasheng.quant.open.gateway.sdk.protocol.MsgType;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufResponseMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.session.Session;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.util.Attribute;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class MessageDecoder extends ByteToMessageDecoder {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) {
        boolean isDecrypt = false;
        MessageHeader header = null;
        byte[] bodyBytes = null;

        try {
            int length = in.readableBytes();
            if (length < 151) {
                // 网络数据未读取完
                return;
            }
            // logger.info("收到数据打印:{}", ByteBufUtil.prettyHexDump(in));

            in.markReaderIndex();
            // 解析请求头
            header = MessageCodecHelper.parseHeader(in);

            length = in.readableBytes();
            if (length < header.getBodyLen()) {
                // 网络数据未读取完
                in.resetReaderIndex();
                return;
            }

            bodyBytes = new byte[header.getBodyLen()];
            in.readBytes(bodyBytes, 0, bodyBytes.length);

            isDecrypt = true;

            // 解析组装完整消息
            SocketMessage socketMsg = MessageCodecHelper.parseBody(header, bodyBytes);
            if (socketMsg != null) {
                // logger.info("Receive notify message, header: {} ", socketMsg.getHeader());
                out.add(socketMsg);
            } else {
                writeErrorResp(ISystemError.ILLEGAL_REQUEST, ctx, header, bodyBytes, isDecrypt);
            }
        } catch (Exception e) {
            logger.error("解码异常", e);
        }
    }

    private void writeErrorResp(ISystemError error, ChannelHandlerContext ctx, MessageHeader header, byte[] bodyBytes,
                                boolean isDecrypt) {
        try {
            if (header != null && MsgType.REQUEST == header.getMsgType()) {
                String requestId = "0";
                if (isDecrypt) {
                    PBRequest protobufMsgBody = PBRequest.parseFrom(bodyBytes);
                    requestId = protobufMsgBody.getRequestId();
                }
                logger.warn("errorCode: {} description: {}", error.getCode(), error.getDescription());
                ctx.writeAndFlush(ProtoBufResponseMessage.buildErrorResp(requestId, header.getSerialNo(),
                        error.getCode(), error.getDescription()));
            } else {
                logger.error("errorCode: {} description: {}", error.getCode(), error.getDescription());
            }
        } catch (Exception e) {
            logger.error("响应异常发生错误 ", e);
        }
    }

}
