package com.huasheng.quant.open.gateway.sdk.netty.codec;

import com.huasheng.quant.open.gateway.sdk.protocol.MessageHeader;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.session.Session;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import io.netty.util.Attribute;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MessageEncoder extends MessageToByteEncoder<SocketMessage> {

	protected final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Override
	public void encode(ChannelHandlerContext ctx, SocketMessage msg, ByteBuf out) {
		String sendMsg = "";
		if (msg.getMsgBody() != null) {
			sendMsg = msg.getMsgBody().toString();
			if (sendMsg.length() > 2048) {
				sendMsg = sendMsg.substring(0, 2000);
			}
		}
		// logger.info("Send request, header: {} msg: {}", msg.getHeader(), sendMsg);
		
		try {
			MessageHeader header = msg.getHeader();

			// 获取包体字节数组
			byte[] bodyBytes = MessageCodecHelper.getBodyBytes(msg);
			if (bodyBytes == null) {
				logger.error("bodyBytes为空");
				return;
			}
			
			Attribute<Session> sessionAttribute = ctx.channel().attr(Session.SESSION_KEY);
			if (sessionAttribute == null || sessionAttribute.get() == null) {
				if (header.getSerialNo() > 0 && StringUtils.isBlank(sessionAttribute.get().getEncryptedKey())) {
					logger.error("动态秘钥为空");
					return;
				}
			}
			
			Session session = sessionAttribute.get();
			
			if (session == null) {
				logger.error("Session为空");
				return;
			}

			// 写入数据
			header.setBodyLen(bodyBytes.length);
			MessageCodecHelper.writeHeader(header, out);
			out.writeBytes(bodyBytes);
		} catch (Exception e) {
			logger.error("编码异常", e);
		}
	}

}
