package com.huasheng.quant.open.gateway.sdk.netty.pipeline;

import com.huasheng.quant.open.gateway.sdk.handlers.Application;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.session.Session;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageCodec;
import io.netty.util.Attribute;
import org.slf4j.Logger;

import java.util.List;

public abstract class AbstractSessionHandler extends MessageToMessageCodec<SocketMessage, SocketMessage> {

	private final Application application;

	protected AbstractSessionHandler(Application application) {
		assert (application != null) : "application is required";
		this.application = application;
	}

	protected void prepareMessageToSend(ChannelHandlerContext ctx, SocketMessage response) {
		prepareMessageToSend(ctx, getSession(ctx), response);
	}

	protected void prepareMessageToSend(ChannelHandlerContext ctx, Session session, SocketMessage response) {
		if (session != null) {
			response.getHeader().setSerialNo(session.getAndIncrementOutgoingMsgSeqNum());
		} else {
			response.getHeader().setSerialNo(0);
		}
		getApplication().beforeSendMessage(ctx, response);
	}

	protected Session getSession(ChannelHandlerContext ctx) {
		Attribute<Session> fixSessionAttribute = ctx.channel().attr(Session.SESSION_KEY);
		return fixSessionAttribute.get();
	}

	protected boolean setSession(ChannelHandlerContext ctx, Session session) {
		assert (session != null) : "Parameter 'fixSession' expected.";
		Attribute<Session> fixSessionAttribute = ctx.channel().attr(Session.SESSION_KEY);
		return fixSessionAttribute.compareAndSet(null, session);
	}

	@Override
	protected void encode(ChannelHandlerContext ctx, SocketMessage msg, List<Object> out) throws Exception {
		prepareMessageToSend(ctx, msg);
		getLogger().trace("Sending outbound: {}", msg);
		out.add(msg);
	}

	@Override
	protected void decode(ChannelHandlerContext ctx, SocketMessage msg, List<Object> out) throws Exception {
		
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		super.exceptionCaught(ctx, cause);
	}

	protected Application getApplication() {
		return application;
	}

	protected abstract Logger getLogger();

}
