package com.huasheng.quant.open.gateway.sdk.netty.pipeline;

import com.huasheng.quant.open.gateway.sdk.handlers.Application;
import com.huasheng.quant.open.gateway.sdk.handlers.DefaultClientApplication;
import com.huasheng.quant.open.gateway.sdk.netty.codec.MessageDecoder;
import com.huasheng.quant.open.gateway.sdk.netty.codec.MessageEncoder;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.handler.codec.MessageToMessageCodec;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

public abstract class SocketChannelInitializer<C extends Channel> extends ChannelInitializer<C> {

	public static final String TAG_DECODER_HANDLER_NAME = "tagDecoder";

	private final EventLoopGroup workerGroup;

	private final Application socketApplication;

	protected SocketChannelInitializer(EventLoopGroup workerGroup, Application socketApplication) {
		this.workerGroup = workerGroup;
		this.socketApplication = socketApplication;
	}

	@Override
	public void initChannel(C ch) throws Exception {
		final ChannelPipeline pipeline = ch.pipeline();
		pipeline.addLast(TAG_DECODER_HANDLER_NAME, new MessageDecoder());
		pipeline.addLast("messageEncoder", new MessageEncoder());
		pipeline.addLast("logging", new LoggingHandler("huasheng", LogLevel.DEBUG));
		pipeline.addLast("session", createSessionHandler());
		if (socketApplication != null) {
			DefaultClientApplication defaultClientApplication = (DefaultClientApplication) socketApplication;
			pipeline.addLast(workerGroup, "app", 
					new DefaultClientApplication(defaultClientApplication.getPushMessageNotifyHandle(), defaultClientApplication.getHsQuantOpenApiHandle())); // process application events events
			pipeline.addBefore(TAG_DECODER_HANDLER_NAME
					, "idle", new IdleStateHandler(3, 5, 0, TimeUnit.SECONDS));
		}
	}

	protected abstract MessageToMessageCodec<SocketMessage, SocketMessage> createSessionHandler();

	protected Application getApplication() {
		return socketApplication;
	}
}
