package com.huasheng.quant.open.gateway.sdk.notify;

import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufNofityMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 * 推送消息处理线程
 */
public abstract class NotifyMessageRunnable implements Runnable {
    protected static final Logger logger = LoggerFactory.getLogger(NotifyMessageRunnable.class);

    private volatile boolean shutdown = false;

    private int threshold;
    private long waitTime;
    private BlockingQueue<ProtoBufNofityMessage> blockingQueue = null;

    /**
     * @param threshold 消费线程池大小
     * @param queueSize 本地内存队列大小
     * @param waitTime  消息写入队列等待时间
     */
    protected NotifyMessageRunnable(int threshold, int queueSize, long waitTime) {
        this.threshold = threshold;
        this.waitTime = waitTime;
        this.blockingQueue = new ArrayBlockingQueue<>(queueSize);
    }

    public int getThreshold() {
        return threshold;
    }

    public void setThreshold(int threshold) {
        this.threshold = threshold;
    }

    /**
     * 消息入队
     *
     * @param protoBufNofityMessage
     */
    public void push(ProtoBufNofityMessage protoBufNofityMessage) {
        try {
            if (!blockingQueue.offer(protoBufNofityMessage, waitTime, TimeUnit.SECONDS)) {
                logger.warn("Push notify message to queue false, queue size: {}.", blockingQueue.size());
            }
        } catch (Exception e) {
            logger.error("Push notify message to queue error, header: {}", protoBufNofityMessage.getHeader(), e);
        }
    }

    @Override
    public void run() {
        while (!shutdown) {
            try {
                ProtoBufNofityMessage notifyMessage = this.blockingQueue.take();
                notifyMessageHandle(notifyMessage);
            } catch (Exception e) {
                logger.error("Consume notify message from queue error, skip message.", e);
                try {
                    TimeUnit.MILLISECONDS.sleep(10);
                } catch (InterruptedException ex) {
                }
            }
        }
    }

    public void shutdown() {
        shutdown = true;
    }

    /**
     * 消息处理方法
     *
     * @param notifyMessage
     */
    public abstract void notifyMessageHandle(ProtoBufNofityMessage notifyMessage);

}
