package com.huasheng.quant.open.gateway.sdk.protocol;

import com.google.protobuf.Any;
import com.google.protobuf.InvalidProtocolBufferException;
import com.google.protobuf.Message;
import com.huasheng.quant.open.gateway.sdk.constant.ISystemError;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.constant.RequestMsgTypeProto.RequestMsgType;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.constant.ResponseMsgTypeProto.ResponseMsgType;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.RequestProto.PBRequest;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.ResponseProto.PBResponse;

import java.util.UUID;

public class ProtoBufRequestMessage extends SocketMessage {

	public ProtoBufRequestMessage() {

	}

	public ProtoBufRequestMessage(RequestMsgType requestMsgType, String token) {
		PBRequest msgBody = PBRequest.newBuilder().setRequestMsgType(requestMsgType).setToken(token)
				.setRequestTime(System.currentTimeMillis()).setRequestId(UUID.randomUUID() + "").build();
		this.getHeader().setMsgType(MsgType.REQUEST);
		this.setMsgBody(msgBody);
	}

	public ProtoBufRequestMessage(RequestMsgType requestMsgType, String token, String requestId, long requestTime) {
		PBRequest msgBody = PBRequest.newBuilder().setRequestMsgType(requestMsgType).setToken(token).setRequestTime(requestTime)
				.setRequestId(requestId).build();
		this.getHeader().setMsgType(MsgType.REQUEST);
		this.setMsgBody(msgBody);
	}

	public long getRequestTime() {
		PBRequest pbRequest = this.getMsgBody();
		return pbRequest.getRequestTime();
	}

	public RequestMsgType getRequestMsgType() {
		PBRequest pbRequest = this.getMsgBody();
		return pbRequest.getRequestMsgType();
	}

	public String getRequestId() {
		PBRequest pbRequest = this.getMsgBody();
		return pbRequest.getRequestId();
	}
	
	public String getToken() {
		PBRequest pbRequest = this.getMsgBody();
		return pbRequest.getToken();
	}

	@Override
	@SuppressWarnings("unchecked")
	public <T> T getPayload(Class<T> cls) {
		PBRequest msgBody = this.getMsgBody();
		if (msgBody == null) {
			return null;
		}
		Class<? extends Message> clsMsgPB = (Class<? extends Message>) cls;
		try {
			return (T) msgBody.getPayload().unpack(clsMsgPB);
		} catch (InvalidProtocolBufferException e) {
			return null;
		}
	}

	@Override
	public void setPayload(Object payLoad) {
		PBRequest pbRequest = this.getMsgBody();
		Any anyPayLoad = Any.pack((Message) payLoad);
		pbRequest = PBRequest.newBuilder(pbRequest).setPayload(anyPayLoad).build();
		this.setMsgBody(pbRequest);

	}

	public ProtoBufResponseMessage buildProtoBufResponse(ResponseMsgType responseMsgType) {
		PBRequest msgBody = this.getMsgBody();
		String requestId = "0";
		if (msgBody != null) {
			requestId = msgBody.getRequestId();
		}
		ProtoBufResponseMessage msg = new ProtoBufResponseMessage();
		msg.getHeader().setMsgType(MsgType.RESPONSE);
		PBResponse pbResponse = PBResponse.newBuilder().setResponseCode(ISystemError.SUCCESS.getCode())
				.setResponseMsg(ISystemError.SUCCESS.getDescription()).setResponseMsgType(responseMsgType)
				.setRequestId(requestId).setResponseTime(System.currentTimeMillis()).build();
		msg.setMsgBody(pbResponse);
		return msg;
	}

}
