package com.huasheng.quant.open.gateway.sdk.protocol;

import com.google.protobuf.Any;
import com.google.protobuf.InvalidProtocolBufferException;
import com.google.protobuf.Message;
import com.huasheng.quant.open.gateway.sdk.constant.ISystemError;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.constant.ResponseMsgTypeProto.ResponseMsgType;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.ResponseProto.PBResponse;
import org.apache.commons.lang3.StringUtils;

public class ProtoBufResponseMessage extends SocketMessage {

	public ProtoBufResponseMessage() {

	}

	public ProtoBufResponseMessage(ResponseMsgType responseMsgType) {
		PBResponse msgBody = PBResponse.newBuilder().setResponseMsgType(responseMsgType)
				.setResponseCode(ISystemError.SUCCESS.getCode()).setRequestId("0")
				.setResponseMsg(ISystemError.SUCCESS.getDescription()).build();
		this.getHeader().setMsgType(MsgType.RESPONSE);
		this.setMsgBody(msgBody);
	}

	public ProtoBufResponseMessage(ResponseMsgType responseMsgType, String responseCode, String responseMsg) {
		PBResponse msgBody = PBResponse.newBuilder().setResponseMsgType(responseMsgType).setResponseCode(responseCode)
				.setRequestId("0").setResponseMsg(responseMsg).build();
		this.getHeader().setMsgType(MsgType.RESPONSE);
		this.setMsgBody(msgBody);
	}

	public void setRequestId(String requestId) {
		PBResponse response = this.getMsgBody();
		response = PBResponse.newBuilder(response).setRequestId(requestId).build();
		this.setMsgBody(response);
	}

	public String getRequestId() {
		PBResponse response = this.getMsgBody();
		return response.getRequestId();
	}

	public void setResponseType(ResponseMsgType responseMsgType) {
		PBResponse response = this.getMsgBody();
		response = PBResponse.newBuilder(response).setResponseMsgType(responseMsgType).build();
		this.setMsgBody(response);
	}

	public ResponseMsgType getResponseType() {
		PBResponse response = this.getMsgBody();
		return response.getResponseMsgType();
	}

	public void setResponseTime(long time) {
		PBResponse response = this.getMsgBody();
		response = PBResponse.newBuilder(response).setResponseTime(time).build();
		this.setMsgBody(response);
	}

	@Override
	@SuppressWarnings("unchecked")
	public <T> T getPayload(Class<T> cls) {
		PBResponse msgBody = this.getMsgBody();
		if (msgBody == null) {
			return null;
		}
		Class<? extends Message> clsMsgPB = (Class<? extends Message>) cls;
		try {
			return (T) msgBody.getPayload().unpack(clsMsgPB);
		} catch (InvalidProtocolBufferException e) {
			return null;
		}
	}

	@Override
	public void setPayload(Object payLoad) {
		PBResponse pbResponse = this.getMsgBody();
		Any anyPayLoad = Any.pack((Message) payLoad);
		pbResponse = PBResponse.newBuilder(pbResponse).setPayload(anyPayLoad).build();
		this.setMsgBody(pbResponse);
	}

	public ProtoBufResponseMessage withError(String errorCode, String errorMsg) {
		PBResponse response = this.getMsgBody();
		response = PBResponse.newBuilder(response).setResponseCode(errorCode).setResponseMsg(errorMsg).build();
		this.setMsgBody(response);
		return this;
	}

	public ProtoBufResponseMessage withError(String errorMsg) {
		PBResponse response = this.getMsgBody();
		response = PBResponse.newBuilder(response).setResponseCode(ISystemError.UNKNOWN_ERROR.getCode())
				.setResponseMsg(errorMsg).build();
		this.setMsgBody(response);
		return this;
	}

	public static ProtoBufResponseMessage buildErrorResp(String requestId, int serialNo, String errorCode,
                                                         String errorMsg) {
		if (StringUtils.isBlank(requestId)) {
			requestId = "0";
		}

		PBResponse pbResponse = PBResponse.newBuilder().setRequestId(requestId)
				.setResponseMsgType(ResponseMsgType.ErrorResponseMsgType).setResponseCode(errorCode)
				.setResponseMsg(errorMsg).setResponseTime(System.currentTimeMillis()).build();

		ProtoBufResponseMessage msg = new ProtoBufResponseMessage();
		msg.getHeader().setMsgType(MsgType.RESPONSE);
		msg.getHeader().setSerialNo(serialNo);
		msg.setMsgBody(pbResponse);
		return msg;
	}

}
