package com.huasheng.quant.open.gateway.sdk.protocol.session;

import io.netty.util.AttributeKey;
import org.apache.commons.lang3.StringUtils;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;

public class Session {
	
	public static final AttributeKey<Session> SESSION_KEY = AttributeKey.valueOf("session");

	private static final AtomicIntegerFieldUpdater<Session> INCOMING_SEQ_NUM_UPDATER = AtomicIntegerFieldUpdater
			.newUpdater(Session.class, "nextIncomingMessageSeqNum");

	private final AtomicInteger nextOutgoingMessageSeqNum = new AtomicInteger();
	private final long createTime;
	private final String clientIp;
	private final String channelId;
	private final String deviceNo;
	private SessionId sessionId;
	private String thirdId;
	private String encryptedKey;
	private String publicKey;
	private String sid;
	private Long memberId;
	private volatile int nextIncomingMessageSeqNum;
	private AtomicInteger lossConnectCount = new AtomicInteger(0);

	private Session(Builder builder) {
		this.clientIp = builder.clientIp;
		this.channelId = builder.channelId;
		this.thirdId = builder.thirdId;
		this.deviceNo = builder.deviceNo;
		this.createTime = builder.createTime;
		this.publicKey = builder.publicKey;
		this.sid = builder.sid;
		if (StringUtils.isNotBlank(thirdId)) {
			this.sessionId = new SessionId(thirdId);
		}
	}

	public static Builder newBuilder() {
		return new Builder();
	}

	public long getCreateTime() {
		return createTime;
	}

	public int getNextOutgoingMessageSeqNum() {
		return nextOutgoingMessageSeqNum.get();
	}

	public void setNextOutgoingMessageSeqNum(int nextOutgoingMessageSeqNum) {
		this.nextOutgoingMessageSeqNum.set(nextOutgoingMessageSeqNum);
	}

	public int getNextIncomingMessageSeqNum() {
		return nextIncomingMessageSeqNum;
	}

	public void setNextIncomingMessageSeqNum(int nextIncomingMessageSeqNum) {
		this.nextIncomingMessageSeqNum = nextIncomingMessageSeqNum;
	}

	public int getAndIncrementOutgoingMsgSeqNum() {
		return nextOutgoingMessageSeqNum.getAndIncrement();
	}

	public String getEncryptedKey() {
		return encryptedKey;
	}

	public void setEncryptedKey(String encryptedKey) {
		this.encryptedKey = encryptedKey;
	}
	
	public String getPublicKey() {
		return publicKey;
	}

	public void setPublicKey(String publicKey) {
		this.publicKey = publicKey;
	}
	
	public String getSid() {
		return sid;
	}

	public void setSid(String sid) {
		this.sid = sid;
	}
	
	public Long getMemberId() {
		return memberId;
	}

	public void setMemberId(Long memberId) {
		this.memberId = memberId;
	}

	public String getClientIp() {
		return clientIp;
	}

	public String getChannelId() {
		return channelId;
	}

	public String getThirdId() {
		return thirdId;
	}
	
	public String getDeviceNo() {
		return deviceNo;
	}

	public int getAndIncrementLossConnectCount() {
		return lossConnectCount.getAndIncrement();
	}

	public void resetLossConnectCount() {
		this.lossConnectCount.set(0);
	}

	public boolean checkAndIncrementIncomingSeqNum(final int num) {
		return INCOMING_SEQ_NUM_UPDATER.compareAndSet(this, num, num + 1);
	}

	public void prepareOutgoing() {

	}
	
	public SessionId getId() {
		return sessionId;
	}

	public static class Builder {

		private String clientIp;
		private String channelId;
		private String thirdId;
		private String deviceNo;
		private long createTime;
		private String publicKey;
		private String sid;

		private Builder() {
		}
		
		public Builder clientIp(String clientIp) {
			this.clientIp = clientIp;
			return this;
		}
		
		public Builder channelId(String channelId) {
			this.channelId = channelId;
			return this;
		}
		
		public Builder thirdId(String thirdId) {
			this.thirdId = thirdId;
			return this;
		}
		
		public Builder deviceNo(String deviceNo) {
			this.deviceNo = deviceNo;
			return this;
		}

		public Builder createTime(long createTime) {
			this.createTime = createTime;
			return this;
		}
		
		public Builder publicKey(String publicKey) {
			this.publicKey = publicKey;
			return this;
		}
		
		public Builder sid(String sid) {
			this.sid = sid;
			return this;
		}

		public Session build() {
			return new Session(this);
		}
	}
}
