package com.huasheng.quant.open.gateway.sdk.utils;

import okhttp3.Headers;
import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.Buffer;
import okio.BufferedSource;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class HttpInterceptor implements Interceptor {
    private final Logger logger = LoggerFactory.getLogger(HttpInterceptor.class);

    @NotNull
    @Override
    public Response intercept(@NotNull Chain chain) throws IOException {
        Request request = chain.request().newBuilder().addHeader("Connection", "close").build();
        RequestBody requestBody = request.body();
        String body = null;
        if (requestBody != null) {
            Buffer buffer = new Buffer();
            requestBody.writeTo(buffer);
            body = buffer.readString(StandardCharsets.UTF_8);
        }
        long start = System.nanoTime();
        try {
            Response response = chain.proceed(request);
            long tookMills = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start);

            ResponseBody responseBody = response.body();
            String rBody = null;
            if (responseBody != null) {
                BufferedSource source = responseBody.source();
                source.request(Long.MAX_VALUE);
                Buffer buffer = source.getBuffer();
                rBody = buffer.clone().readString(StandardCharsets.UTF_8);
            }
            if (logger.isInfoEnabled()) {
                logger.info("请求HTTP接口---请求方式: {}, 链接: {}, 耗时: {}ms, 参数: {}, 响应: {}",
                        request.method(), request.url(), tookMills, body, rBody);
            }
            return response;
        } catch (Exception e) {
            long tookMills = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start);
            logger.info("请求HTTP接口发生异常---请求方式: {}, 链接: {}, 耗时: {}ms, 参数: {}",
                    request.method(), request.url(), tookMills, body);
            throw e;
        }
    }
}
