package com.huasheng.quant.open.gateway.sdk.vo.trade;

import com.huasheng.quant.open.gateway.sdk.constant.trade.TradeCondStatus;
import com.huasheng.quant.open.gateway.sdk.constant.trade.TradeCondTrackType;
import com.huasheng.quant.open.gateway.sdk.constant.trade.EntrustBs;
import com.huasheng.quant.open.gateway.sdk.constant.trade.EntrustType;
import com.huasheng.quant.open.gateway.sdk.constant.trade.ExchangeType;
import com.huasheng.quant.open.gateway.sdk.constant.trade.SessionType;

import java.io.Serializable;

public class CondOrder implements Serializable {

    private static final long serialVersionUID = -1438068356460815061L;
    
    /**
     * 条件单号
     */
    private String condOrderId;

    /**
     * 股票dataType
     */
    private String dataType;

    /**
     * 股票代码
     */
    private String stockCode;

    /**
     * 股票名称
     */
    private String stockName;

    /**
     * 股票名称-繁体
     */
    private String stockNameTc;

    /**
     * 股票名称-英文
     */
    private String stockNameEn;

    /**
     * 市场代码
     *
     * @see ExchangeType
     */
    private String exchangeType;

    /**
     * 31止盈限价单 32止盈市价单(美股) 33止损限价单 34止损市价单(美股) 35追踪止损限价单 36追踪止损市价单(美股)
     *
     * @see EntrustType
     */
    private String entrustType;

    /**
     * 3只支持盘中 6美股只支持盘前盘后 7美股支持盘中及盘前盘后
     *
     * @see SessionType
     */
    private String sessionType;

    /**
     * 条件单状态  1:待触发 2:已触发 3:暂停 4:已过期 5:已删除 6:错误 8:止盈止损单失效 9:除权除息失效
     *
     * @see TradeCondStatus
     */
    private String status;

    /**
     * 是否可以撤单 0否 1是
     */
    private String canBeCancel;

    /**
     * 是否可以改单 0否 1是
     */
    private String canBeModify;

    /**
     * 买卖方向  '1'-多头开仓、'2'-多头平仓、'3'-空头平仓、'4'-空头开仓
     *
     * @see EntrustBs
     */
    private String entrustBs;

    /**
     * 委托数量
     */
    private String entrustAmount;

    /**
     * 下单时间
     */
    private String createTime;

    /**
     * 启动时间 空为立即执行
     */
    private String startTime;

    /**
     * 到期时间 为空当天24点到期
     */
    private String endTime;

    /**
     * 错误码
     */
    private String errorCode;

    /**
     * 异常信息
     */
    private String errorMsg;

    /**
     * 条件值 价格/百分比值
     */
    private String condValue;

    /**
     * 条件指定价
     */
    private String condPrice;

    /**
     * 条件跟踪类型 1百分比、2价差、3价格
     *
     * @see TradeCondTrackType
     */
    private String condTrackType;

    public String getCondOrderId() {
        return condOrderId;
    }

    public void setCondOrderId(String condOrderId) {
        this.condOrderId = condOrderId;
    }

    public String getDataType() {
        return dataType;
    }

    public void setDataType(String dataType) {
        this.dataType = dataType;
    }

    public String getStockCode() {
        return stockCode;
    }

    public void setStockCode(String stockCode) {
        this.stockCode = stockCode;
    }

    public String getStockName() {
        return stockName;
    }

    public void setStockName(String stockName) {
        this.stockName = stockName;
    }

    public String getStockNameTc() {
        return stockNameTc;
    }

    public void setStockNameTc(String stockNameTc) {
        this.stockNameTc = stockNameTc;
    }

    public String getStockNameEn() {
        return stockNameEn;
    }

    public void setStockNameEn(String stockNameEn) {
        this.stockNameEn = stockNameEn;
    }

    public String getExchangeType() {
        return exchangeType;
    }

    public void setExchangeType(String exchangeType) {
        this.exchangeType = exchangeType;
    }

    public String getEntrustType() {
        return entrustType;
    }

    public void setEntrustType(String entrustType) {
        this.entrustType = entrustType;
    }

    public String getSessionType() {
        return sessionType;
    }

    public void setSessionType(String sessionType) {
        this.sessionType = sessionType;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getCanBeCancel() {
        return canBeCancel;
    }

    public void setCanBeCancel(String canBeCancel) {
        this.canBeCancel = canBeCancel;
    }

    public String getCanBeModify() {
        return canBeModify;
    }

    public void setCanBeModify(String canBeModify) {
        this.canBeModify = canBeModify;
    }

    public String getEntrustBs() {
        return entrustBs;
    }

    public void setEntrustBs(String entrustBs) {
        this.entrustBs = entrustBs;
    }

    public String getEntrustAmount() {
        return entrustAmount;
    }

    public void setEntrustAmount(String entrustAmount) {
        this.entrustAmount = entrustAmount;
    }

    public String getCreateTime() {
        return createTime;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public String getStartTime() {
        return startTime;
    }

    public void setStartTime(String startTime) {
        this.startTime = startTime;
    }

    public String getEndTime() {
        return endTime;
    }

    public void setEndTime(String endTime) {
        this.endTime = endTime;
    }

    public String getErrorCode() {
        return errorCode;
    }

    public void setErrorCode(String errorCode) {
        this.errorCode = errorCode;
    }

    public String getErrorMsg() {
        return errorMsg;
    }

    public void setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
    }

    public String getCondValue() {
        return condValue;
    }

    public void setCondValue(String condValue) {
        this.condValue = condValue;
    }

    public String getCondPrice() {
        return condPrice;
    }

    public void setCondPrice(String condPrice) {
        this.condPrice = condPrice;
    }

    public String getCondTrackType() {
        return condTrackType;
    }

    public void setCondTrackType(String condTrackType) {
        this.condTrackType = condTrackType;
    }

}
