package com.huasheng.quant.open.gateway.api;

import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.huasheng.quant.open.gateway.sdk.constant.RateType;
import com.huasheng.quant.open.gateway.sdk.constant.algo.AlgoActionType;
import com.huasheng.quant.open.gateway.sdk.constant.algo.AlgoEntrustType;
import com.huasheng.quant.open.gateway.sdk.constant.algo.AlgoStrategySensitivityType;
import com.huasheng.quant.open.gateway.sdk.constant.algo.AlgoStrategyType;
import com.huasheng.quant.open.gateway.sdk.constant.futures.FuturesEntrustBs;
import com.huasheng.quant.open.gateway.sdk.constant.futures.FuturesEntrustType;
import com.huasheng.quant.open.gateway.sdk.constant.futures.FuturesValidTimeType;
import com.huasheng.quant.open.gateway.sdk.constant.hq.*;
import com.huasheng.quant.open.gateway.sdk.constant.trade.*;
import com.huasheng.quant.open.gateway.sdk.domain.ModelResult;
import com.huasheng.quant.open.gateway.sdk.notify.NotifyMessageRunnable;
import com.huasheng.quant.open.gateway.sdk.vo.CommonStringVo;
import com.huasheng.quant.open.gateway.sdk.vo.algo.*;
import com.huasheng.quant.open.gateway.sdk.vo.futures.*;
import com.huasheng.quant.open.gateway.sdk.vo.hq.*;
import com.huasheng.quant.open.gateway.sdk.vo.trade.*;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;

/**
 * 量化OpenAPI接入Demo程序
 */
public class HSQuantOpenApiTest {
    private static final Logger logger = LoggerFactory.getLogger(HSQuantOpenApiTest.class);

    public static void main(String[] args) throws Exception {
        // 请求IP（* 开发者部署 HS Gateway 所在服务器IP *）
        String ip = "127.0.0.1";
        // 业务请求端口（* 开发者部署 HS Gateway 设置的HTTP端口 *）
        int httpPort = 11111;
        // 消息推送端口（* 开发者部署 HS Gateway 设置的TCP端口 *）
        int tcpPort = 11112;

        // 消息推送处理类
        PushNotifyMessageHandler notifyHandle = new PushNotifyMessageHandler(2, 10000, 1);

        // 初始化API demo对象
        HSQuantOpenTradeAndHqApiDemo hsQuantOpenTradeAndHqApiDemo = new HSQuantOpenTradeAndHqApiDemo(ip, httpPort, tcpPort, notifyHandle);
        boolean start = hsQuantOpenTradeAndHqApiDemo.start();
        if (!start) { // 客户端启动失败
            System.exit(0);
        }

        if (hsQuantOpenTradeAndHqApiDemo.isActive()) {
            while (true) {
                System.out.println("====================================================");
                System.out.println("HTTP 请求地址   " + "http://" + ip + ":" + String.valueOf(httpPort));
                System.out.println("TCP 长连接地址  " + ip + ":" + String.valueOf(tcpPort));
                System.out.println("OpenAPI在线文档 https://quant-open.hstong.com/api-docs/");
                System.out.println("接口名: queryRateList                     接口描述: 综合接口-查询汇率信息");
                System.out.println("接口名: tradeLogin                        接口描述: 交易接口-交易登录");
                System.out.println("接口名: tradeLogout                       接口描述: 交易接口-交易登出");
                System.out.println("接口名: queryMarginFundInfo               接口描述: 交易接口-查询客户资金信息");
                System.out.println("接口名: queryHoldsList                    接口描述: 交易接口-查询持仓列表");
                System.out.println("接口名: queryMaxBuyAmount                 接口描述: 交易接口-查询最大可买");
                System.out.println("接口名: queryMaxSellAmount                接口描述: 交易接口-查询最大可卖");
                System.out.println("接口名: queryRealFundJourList             接口描述: 交易接口-查询当日资金流水");
                System.out.println("接口名: queryHistoryFundJourList          接口描述: 交易接口-查询历史资金流水");
                System.out.println("接口名: entrustChangeCancel               接口描述: 交易接口-下单/改单/撤单");
                System.out.println("接口名: entrust                           接口描述: 交易接口-下单");
                System.out.println("接口名: changeEntrust                     接口描述: 交易接口-改单");
                System.out.println("接口名: cancelEntrust                     接口描述: 交易接口-撤单");
                System.out.println("接口名: entrustCond                       接口描述: 交易接口-条件单-下单/改单/撤单");
                System.out.println("接口名: batchCancelEntrust                接口描述: 交易接口-批量/撤单");
                System.out.println("接口名: queryMaxAvailableAsset            接口描述: 交易接口-查询最大可用资产");
                System.out.println("接口名: queryRealEntrustList              接口描述: 交易接口-查询当日委托");
                System.out.println("接口名: queryHistoryEntrustList           接口描述: 交易接口-查询历史委托");
                System.out.println("接口名: queryRealDeliverList              接口描述: 交易接口-查询当日成交");
                System.out.println("接口名: queryHistoryDeliverList           接口描述: 交易接口-查询历史成交");
                System.out.println("接口名: queryRealCondOrderList            接口描述: 交易接口-查询当日条件单");
                System.out.println("接口名: queryHistoryCondOrderList         接口描述: 交易接口-查询历史条件单");
                System.out.println("接口名: queryBeforeAndAfterSupport        接口描述: 交易接口-查询是否支持盘前盘后交易");
                System.out.println("接口名: queryStockShortInfo               接口描述: 交易接口-查询股票沽空信息");
                System.out.println("接口名: queryMarginFullInfo               接口描述: 交易接口-查询股票融资融券数据");
                System.out.println("接口名: subscribe                         接口描述: 交易接口-订阅交易推送");
                System.out.println("接口名: unsubscribe                       接口描述: 交易接口-取消订阅交易推送");
                System.out.println("接口名: futuresQueryFundInfo              接口描述: 期货接口-查询客户资金信息");
                System.out.println("接口名: futuresQueryHoldsList             接口描述: 期货接口-查询持仓列表");
                System.out.println("接口名: futuresQueryMaxBuySellAmount      接口描述: 期货接口-查询最大可买可卖");
                System.out.println("接口名: futuresQueryProductInfo           接口描述: 期货接口-查询合约产品信息");
                System.out.println("接口名: futuresEntrust                    接口描述: 期货接口-下单/改单/撤单");
                System.out.println("接口名: futuresQueryRealEntrustList       接口描述: 期货接口-查询当日委托");
                System.out.println("接口名: futuresQueryHistoryEntrustList    接口描述: 期货接口-查询历史委托");
                System.out.println("接口名: futuresQueryRealDeliverList       接口描述: 期货接口-查询当日成交");
                System.out.println("接口名: futuresQueryHistoryDeliverList    接口描述: 期货接口-查询历史成交");
                System.out.println("接口名: algoEntrust                       接口描述: 策略单接口-下单");
                System.out.println("接口名: algoCancelOrder                   接口描述: 策略单接口-撤销母单");
                System.out.println("接口名: algoCancelSubOrder                接口描述: 策略单接口-撤销子单");
                System.out.println("接口名: algoModifyOrder                   接口描述: 策略单接口-改单");
                System.out.println("接口名: algoOrderAction                   接口描述: 策略单接口-操作");
                System.out.println("接口名: algoQueryOrderList                接口描述: 策略单接口-查询母单列表");
                System.out.println("接口名: algoQuerySubOrderIdList           接口描述: 策略单接口-查询子单");
                System.out.println("接口名: queryHqBasicQot                   接口描述: 行情接口-查询股票基础报价");
                System.out.println("接口名: queryHqBroker                     接口描述: 行情接口-查询买卖经纪摆盘");
                System.out.println("接口名: queryHqOrderBook                  接口描述: 行情接口-查询买卖档");
                System.out.println("接口名: queryHqTicker                     接口描述: 行情接口-查询逐笔");
                System.out.println("接口名: queryHqKLine                      接口描述: 行情接口-查询K线");
                System.out.println("接口名: queryHqTimeShare                  接口描述: 行情接口-查询分时");
                System.out.println("接口名: queryHqUsOptionChainExpireDate    接口描述: 行情接口-查询期权链到期日列表");
                System.out.println("接口名: queryHqUsOptionChainCode          接口描述: 行情接口-查询期权链Code列表");
                System.out.println("接口名: hqSubscribeBasicQot               接口描述: 行情接口-订阅基础报价推送消息");
                System.out.println("接口名: hqSubscribeBroker                 接口描述: 行情接口-订阅买卖经纪推送消息");
                System.out.println("接口名: hqSubscribeOrderBook              接口描述: 行情接口-订阅买卖档推送消息");
                System.out.println("接口名: hqSubscribeTicker                 接口描述: 行情接口-订阅逐笔推送消息");
                System.out.println("接口名: hqSubscribeArcaBook               接口描述: 行情接口-订阅深度摆盘-ARCABOOK买卖档推送消息");
                System.out.println("接口名: hqSubscribeTotalView              接口描述: 行情接口-订阅深度摆盘-TOTALVIEW买卖档推送消息");
                System.out.println("接口名: hqUnsubscribeBasicQot             接口描述: 行情接口-取消订阅基础报价推送消息");
                System.out.println("接口名: hqUnsubscribeBroker               接口描述: 行情接口-取消订阅买卖经纪推送消息");
                System.out.println("接口名: hqUnsubscribeOrderBook            接口描述: 行情接口-取消订阅买卖档推送消息");
                System.out.println("接口名: hqUnsubscribeTicker               接口描述: 行情接口-取消订阅逐笔推送消息");
                System.out.println("接口名: hqUnsubscribeArcaBook             接口描述: 行情接口-取消订阅深度摆盘-ARCABOOK买卖档推送消息");
                System.out.println("接口名: hqUnsubscribeTotalView            接口描述: 行情接口-取消订阅深度摆盘-TOTALVIEW买卖档推送消息");
                System.out.println("接口名: stop                              接口描述: ！！！程序退出，该函数将退出程序，并断开TCP链接！！！");
                System.out.println("====================================================");

                Scanner sc = new Scanner(System.in);
                System.out.print("请输入需要执行的接口名: ");
                String method_name = sc.nextLine();  // 读取字符串型输入

                switch (method_name) {
                    case "queryRateList": {
                        hsQuantOpenTradeAndHqApiDemo.queryRateList();
                        break;
                    }
                    case "tradeLogin": {
                        hsQuantOpenTradeAndHqApiDemo.tradeLogin();
                        break;
                    }
                    case "tradeLogout": {
                        hsQuantOpenTradeAndHqApiDemo.tradeLogout();
                        break;
                    }
                    case "queryMarginFundInfo": {
                        hsQuantOpenTradeAndHqApiDemo.queryMarginFundInfo();
                        break;
                    }
                    case "queryHoldsList": {
                        hsQuantOpenTradeAndHqApiDemo.queryHoldsList();
                        break;
                    }
                    case "queryMaxBuyAmount": {
                        hsQuantOpenTradeAndHqApiDemo.queryMaxBuyAmount();
                        break;
                    }
                    case "queryMaxSellAmount": {
                        hsQuantOpenTradeAndHqApiDemo.queryMaxSellAmount();
                        break;
                    }
                    case "queryRealFundJourList": {
                        hsQuantOpenTradeAndHqApiDemo.queryRealFundJourList();
                        break;
                    }
                    case "queryHistoryFundJourList": {
                        hsQuantOpenTradeAndHqApiDemo.queryHistoryFundJourList();
                        break;
                    }
                    case "entrustChangeCancel": {
                        hsQuantOpenTradeAndHqApiDemo.entrustChangeCancel();
                        break;
                    }
                    case "entrust": {
                        hsQuantOpenTradeAndHqApiDemo.entrust();
                        break;
                    }
                    case "changeEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.changeEntrust();
                        break;
                    }
                    case "cancelEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.cancelEntrust();
                        break;
                    }
                    case "batchCancelEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.batchCancelEntrust();
                        break;
                    }
                    case "entrustCond": {
                        hsQuantOpenTradeAndHqApiDemo.entrustCond();
                        break;
                    }
                    case "queryMaxAvailableAsset": {
                        hsQuantOpenTradeAndHqApiDemo.queryMaxAvailableAsset();
                        break;
                    }
                    case "queryRealEntrustList": {
                        hsQuantOpenTradeAndHqApiDemo.queryRealEntrustList();
                        break;
                    }
                    case "queryHistoryEntrustList": {
                        hsQuantOpenTradeAndHqApiDemo.queryHistoryEntrustList();
                        break;
                    }
                    case "queryRealDeliverList": {
                        hsQuantOpenTradeAndHqApiDemo.queryRealDeliverList();
                        break;
                    }
                    case "queryHistoryDeliverList": {
                        hsQuantOpenTradeAndHqApiDemo.queryHistoryDeliverList();
                        break;
                    }
                    case "queryRealCondOrderList": {
                        hsQuantOpenTradeAndHqApiDemo.queryRealCondOrderList();
                        break;
                    }
                    case "queryHistoryCondOrderList": {
                        hsQuantOpenTradeAndHqApiDemo.queryHistoryCondOrderList();
                        break;
                    }
                    case "queryBeforeAndAfterSupport": {
                        hsQuantOpenTradeAndHqApiDemo.queryBeforeAndAfterSupport();
                        break;
                    }
                    case "queryStockShortInfo": {
                        hsQuantOpenTradeAndHqApiDemo.queryStockShortInfo();
                        break;
                    }
                    case "queryMarginFullInfo": {
                        hsQuantOpenTradeAndHqApiDemo.queryMarginFullInfo();
                        break;
                    }
                    case "subscribe": {
                        hsQuantOpenTradeAndHqApiDemo.subscribe();
                        break;
                    }
                    case "unsubscribe": {
                        hsQuantOpenTradeAndHqApiDemo.unsubscribe();
                        break;
                    }
                    case "futuresQueryFundInfo": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryFundInfo();
                        break;
                    }
                    case "futuresQueryHoldsList": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryHoldsList();
                        break;
                    }
                    case "futuresQueryMaxBuySellAmount": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryMaxBuySellAmount();
                        break;
                    }
                    case "futuresQueryProductInfo": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryProductInfo();
                        break;
                    }
                    case "futuresEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.futuresEntrust();
                        break;
                    }
                    case "futuresModifyEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.futuresModifyEntrust();
                        break;
                    }
                    case "futuresCancelEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.futuresCancelEntrust();
                        break;
                    }
                    case "futuresQueryRealEntrustList": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryRealEntrustList();
                        break;
                    }
                    case "futuresQueryHistoryEntrustList": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryHistoryEntrustList();
                        break;
                    }
                    case "futuresQueryRealDeliverList": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryRealDeliverList();
                        break;
                    }
                    case "futuresQueryHistoryDeliverList": {
                        hsQuantOpenTradeAndHqApiDemo.futuresQueryHistoryDeliverList();
                        break;
                    }
                    case "algoEntrust": {
                        hsQuantOpenTradeAndHqApiDemo.algoEntrust();
                        break;
                    }
                    case "algoCancelOrder": {
                        hsQuantOpenTradeAndHqApiDemo.algoCancelOrder();
                        break;
                    }
                    case "algoCancelSubOrder": {
                        hsQuantOpenTradeAndHqApiDemo.algoCancelSubOrder();
                        break;
                    }
                    case "algoModifyOrder": {
                        hsQuantOpenTradeAndHqApiDemo.algoModifyOrder();
                        break;
                    }
                    case "algoOrderAction": {
                        hsQuantOpenTradeAndHqApiDemo.algoOrderAction();
                        break;
                    }
                    case "algoQueryOrderList": {
                        hsQuantOpenTradeAndHqApiDemo.algoQueryOrderList();
                        break;
                    }
                    case "algoQuerySubOrderIdList": {
                        hsQuantOpenTradeAndHqApiDemo.algoQuerySubOrderIdList();
                        break;
                    }
                    case "queryHqBasicQot": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqBasicQot();
                        break;
                    }
                    case "queryHqBroker": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqBroker();
                        break;
                    }
                    case "queryHqOrderBook": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqOrderBook();
                        break;
                    }
                    case "queryHqTicker": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqTicker();
                        break;
                    }
                    case "queryHqKLine": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqKLine();
                        break;
                    }
                    case "queryHqTimeShare": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqTimeShare();
                        break;
                    }
                    case "queryHqUsOptionChainExpireDate": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqUsOptionChainExpireDate();
                        break;
                    }
                    case "queryHqUsOptionChainCode": {
                        hsQuantOpenTradeAndHqApiDemo.queryHqUsOptionChainCode();
                        break;
                    }
                    case "hqSubscribeBasicQot": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeBasicQot();
                        break;
                    }
                    case "hqSubscribeBroker": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeBroker();
                        break;
                    }
                    case "hqSubscribeOrderBook": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeOrderBook();
                        break;
                    }
                    case "hqSubscribeTicker": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeTicker();
                        break;
                    }
                    case "hqSubscribeArcaBook": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeArcaBook();
                        break;
                    }
                    case "hqSubscribeTotalView": {
                        hsQuantOpenTradeAndHqApiDemo.hqSubscribeTotalView();
                        break;
                    }
                    case "hqUnsubscribeBasicQot": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeBasicQot();
                        break;
                    }
                    case "hqUnsubscribeBroker": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeBroker();
                        break;
                    }
                    case "hqUnsubscribeOrderBook": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeOrderBook();
                        break;
                    }
                    case "hqUnsubscribeTicker": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeTicker();
                        break;
                    }
                    case "hqUnsubscribeArcaBook": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeArcaBook();
                        break;
                    }
                    case "hqUnsubscribeTotalView": {
                        hsQuantOpenTradeAndHqApiDemo.hqUnsubscribeTotalView();
                        break;
                    }
                    case "stop": {
                        hsQuantOpenTradeAndHqApiDemo.stop();
                        hsQuantOpenTradeAndHqApiDemo.sleep();
                        // 退出系统
                        System.exit(1);
                    }
                    default:
                        System.out.println("接口名输入有误，请参考提示重新输入！");
                }
                hsQuantOpenTradeAndHqApiDemo.sleep();
                //  输入回车,才往下走
                System.out.println("请输入回车，继续输入下一个接口");
                sc.nextLine();
            }
        }
    }

    static class HSQuantOpenTradeAndHqApiDemo {
        private int sleepTime = 500;

        private HSQuantOpenApiHandle quantOpenApiHandle = null;

        private Gson gson;

        public HSQuantOpenTradeAndHqApiDemo(String ip, int httpPort, int tcpPort, NotifyMessageRunnable notifyHandle) {
            this.quantOpenApiHandle = new HSQuantOpenApiHandle(ip, httpPort, tcpPort, notifyHandle);
            this.gson = new GsonBuilder().create();
        }

        /**
         * 对象初始化
         */
        public boolean start() throws Exception {
            // 注册回调
            addShutdownHook();

            // [启动客户端连接]
            return this.quantOpenApiHandle.startConnect();
        }

        public boolean isActive() {
            return quantOpenApiHandle.isActive();
        }

        /**
         * 交易登录
         */
        public void tradeLogin() {
            /* 交易密码需要使用AES加密（ECB/PKCSPadding），AES秘钥为"m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY"，需要进行Base64处理
             * 加密步骤：Base64.Encode(AES.Encrypt(Base64.Decode("m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY"), "123456"))
             * 如密码为"123456"，加密后密码："W1U8iZIppSE+mBMtzy9vZQ=="
             */
            TradeLoginParam tradeLoginParam = new TradeLoginParam();
            tradeLoginParam.setPassword("123456");
            tradeLoginParam.setKeyBase64("m+qS04/2CH1OweCnmXZ3TDZkCQS+hBzY");

            boolean b = quantOpenApiHandle.tradeLogin(tradeLoginParam);
            System.out.println("交易登录结果: " + b);
        }

        /**
         * 交易登出
         */
        public void tradeLogout() {
            boolean b = quantOpenApiHandle.tradeLogout();
            System.out.println("交易登出结果: " + b);
        }

        /**
         * 查询汇率信息
         */
        public void queryRateList() {
            quantOpenApiHandle.queryRateList(RateType.EXCHANGE);
        }

        /**
         * 查询客户资金信息
         */
        public void queryMarginFundInfo() {
            ExchangeTypeParam exchangeTypeParam = new ExchangeTypeParam();
            exchangeTypeParam.setExchangeType(ExchangeType.K.getCode());

            quantOpenApiHandle.queryMarginFundInfo(exchangeTypeParam);
        }

        /**
         * 查询持仓列表
         */
        public void queryHoldsList() {
            ExchangeTypeParam exchangeTypeParam = new ExchangeTypeParam();
            exchangeTypeParam.setExchangeType(ExchangeType.K.getCode());

            quantOpenApiHandle.queryHoldsList(exchangeTypeParam);
        }

        /**
         * 查询最大可买
         */
        public void queryMaxBuyAmount() {
            MaxBuySellAmountParam maxBuySellAmountParam = new MaxBuySellAmountParam();
            maxBuySellAmountParam.setExchangeType(ExchangeType.K.getCode());
            maxBuySellAmountParam.setStockCode("00700.HK");
            maxBuySellAmountParam.setEntrustPrice("295.11");
            maxBuySellAmountParam.setClientType(ClientType.INTERNET.getCode());

            quantOpenApiHandle.queryMaxBuyAmount(maxBuySellAmountParam);
        }

        /**
         * 查询最大可卖
         */
        public void queryMaxSellAmount() {
            MaxBuySellAmountParam maxBuySellAmountParam = new MaxBuySellAmountParam();
            maxBuySellAmountParam.setExchangeType(ExchangeType.K.getCode());
            maxBuySellAmountParam.setStockCode("00700.HK");
            maxBuySellAmountParam.setClientType(ClientType.INTERNET.getCode());

            quantOpenApiHandle.queryMaxSellAmount(maxBuySellAmountParam);
        }

        /**
         * 查询当日资金流水
         */
        public void queryRealFundJourList() {
            RealPageQueryParam realPageQueryParam = new RealPageQueryParam();
            realPageQueryParam.setExchangeType(ExchangeType.K.getCode());
            realPageQueryParam.setQueryParamStr("0");
            realPageQueryParam.setQueryCount(20);

            quantOpenApiHandle.queryRealFundJourList(realPageQueryParam);
        }

        /**
         * 查询历史资金流水
         */
        public void queryHistoryFundJourList() {
            HistoryPageIntQueryParam historyPageQueryParam = new HistoryPageIntQueryParam();
            historyPageQueryParam.setExchangeType(ExchangeType.K.getCode());
            historyPageQueryParam.setQueryParamStr("0");
            historyPageQueryParam.setQueryCount(10);
            historyPageQueryParam.setStartDate(20231101);
            historyPageQueryParam.setEndDate(20231104);

            quantOpenApiHandle.queryHistoryFundJourList(historyPageQueryParam);
        }

        /**
         * 期货-查询客户资金信息
         */
        public void futuresQueryFundInfo() {
            quantOpenApiHandle.futuresQueryFundInfo();
        }

        /**
         * 期货-查询持仓列表
         */
        public void futuresQueryHoldsList() {
            quantOpenApiHandle.futuresQueryHoldsList();
        }

        /**
         * 期货-查询最大可买可卖
         */
        public void futuresQueryMaxBuySellAmount() {
            FuturesCodeParam futuresCodeParam = new FuturesCodeParam();
            futuresCodeParam.setStockCode("MHI2312");

            quantOpenApiHandle.futuresQueryMaxBuySellAmount(futuresCodeParam);
        }

        /**
         * 期货-查询合约产品信息
         */
        public void futuresQueryProductInfo() {
            FuturesCodeListParam futuresCodeListParam = new FuturesCodeListParam(Lists.newArrayList("HSI2312"));
            quantOpenApiHandle.futuresQueryProductInfo(futuresCodeListParam);
        }

        /**
         * 下单/改单/撤单
         */
        public void entrustChangeCancel() {
            String exchangeType = ExchangeType.K.getCode();
            String stockCode = "00700.HK";
            EntrustParam tradeEntrustParam = getEntrustParam(stockCode, exchangeType);
            // 冰山单披露数量
            // tradeEntrustParam.setIceBergDisplaySize("10");

            // 委托下单
            ModelResult<CommonStringVo> voModelResult = quantOpenApiHandle.entrust(tradeEntrustParam);

            if (voModelResult.isSuccess() && voModelResult.getModel() != null) {
                CommonStringVo commonStringVo = voModelResult.getModel();
                String entrustId = commonStringVo.getData();

                ChangeEntrustParam changeEntrustParam = getChangeEntrustParam(entrustId, stockCode, exchangeType);

                // 执行改单
                ModelResult<CommonStringVo> changeEntrustModelResult = quantOpenApiHandle.changeEntrust(changeEntrustParam);
                if (changeEntrustModelResult.isSuccess() && changeEntrustModelResult.getModel() != null) {

                    CancelEntrustParam cancelEntrustParam = new CancelEntrustParam();
                    cancelEntrustParam.setEntrustId(entrustId);
                    cancelEntrustParam.setStockCode(stockCode);
                    cancelEntrustParam.setExchangeType(exchangeType);

                    // 执行撤单
                    ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.cancelEntrust(cancelEntrustParam);
                }
            }
        }

        /**
         * 下单
         */
        public void entrust() {
            EntrustParam entrustParam = getEntrustParam("09988.HK", ExchangeType.K.getCode());
            ModelResult<CommonStringVo> voModelResult = quantOpenApiHandle.entrust(entrustParam);
            if (voModelResult.isSuccess() && voModelResult.getModel() != null) {
                CommonStringVo commonStringVo = voModelResult.getModel();
                String entrustId = commonStringVo.getData();
                System.out.println("entrustId: " + entrustId);
            }
        }

        /**
         * 改单
         */
        public void changeEntrust() {
            String entrustId = "5630172";
            String stockCode = "00700.HK";
            String exchangeType = ExchangeType.K.getCode();
            ChangeEntrustParam changeEntrustParam = getChangeEntrustParam(entrustId, stockCode, exchangeType);

            // 执行改单
            ModelResult<CommonStringVo> changeEntrustModelResult = quantOpenApiHandle.changeEntrust(changeEntrustParam);
        }

        /**
         * 撤单
         */
        public void cancelEntrust() {
            CancelEntrustParam cancelEntrustParam = new CancelEntrustParam();
            cancelEntrustParam.setEntrustId("5630130");
            cancelEntrustParam.setStockCode("00700.HK");
            cancelEntrustParam.setExchangeType(ExchangeType.K.getCode());

            // 执行撤单
            ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.cancelEntrust(cancelEntrustParam);
        }

        @NotNull
        private static EntrustParam getEntrustParam(String stockCode, String exchangeType) {
            String entrustPrice = "81";
            String entrustAmount = "200";

            EntrustParam tradeEntrustParam = new EntrustParam();
            tradeEntrustParam.setStockCode(stockCode);
            tradeEntrustParam.setExchangeType(exchangeType);
            // 港股 ：'0'-竞价限价、'1'-竞价、'2'-增强限价盘、'3'-限价盘、'4'-特别限价盘、'6'-暗盘
            // 美股 ：'3'-限价盘、'5'-市价盘、'8'-冰山市价、'9'-冰山限价、'10'-隐藏市价、'11'-隐藏限价
            // A股  ：'3'-限价盘
            tradeEntrustParam.setEntrustType(EntrustType.LIMIT.getCode());
            tradeEntrustParam.setEntrustPrice(entrustPrice);
            tradeEntrustParam.setEntrustAmount(entrustAmount);
            // '1'-多头开仓、'2'-多头平仓、'3'-空头平仓、'4'-空头开仓
            tradeEntrustParam.setEntrustBs(EntrustBs.BUY.getCode());
            // 交易所： "ARCA" "SMART"....
            tradeEntrustParam.setExchange(EntrustEx.ARCA);
            // 是否盘前盘后交易
            tradeEntrustParam.setSessionType(SessionType.GENERAL_ORDER_NO.getCode());
            return tradeEntrustParam;
        }

        @NotNull
        private static ChangeEntrustParam getChangeEntrustParam(String entrustId, String stockCode, String exchangeType) {
            String changeEntrustPrice = "301";
            String changeEntrustAmount = "2000";

            ChangeEntrustParam changeEntrustParam = new ChangeEntrustParam();
            changeEntrustParam.setEntrustId(entrustId);
            changeEntrustParam.setStockCode(stockCode);
            changeEntrustParam.setExchangeType(exchangeType);
            changeEntrustParam.setEntrustPrice(changeEntrustPrice);
            changeEntrustParam.setEntrustAmount(changeEntrustAmount);
            return changeEntrustParam;
        }

        /**
         * 批量撤单
         */
        public void batchCancelEntrust() {
            // 交易市场类型: 'K'港股、'P'美股、'v'深股通、't'沪股通
            // String exchangeType = ExchangeType.K.getCode();
            // String stockCode = "00700.HK";
            // String entrustPrice = "345";
            // String entrustAmount = "1000";
            //
            // EntrustParam tradeEntrustParam = new EntrustParam();
            // tradeEntrustParam.setStockCode(stockCode);
            // tradeEntrustParam.setExchangeType(exchangeType);
            // // 港股 ：'0'-竞价限价、'1'-竞价、'2'-增强限价盘、'3'-限价盘、'4'-特别限价盘、'6'-暗盘
            // // 美股 ：'3'-限价盘、'5'-市价盘、'8'-冰山市价、'9'-冰山限价、'10'-隐藏市价、'11'-隐藏限价
            // // A股  ：'3'-限价盘
            // tradeEntrustParam.setEntrustType(EntrustType.AUCTION.getCode());
            // tradeEntrustParam.setEntrustPrice(entrustPrice);
            // tradeEntrustParam.setEntrustAmount(entrustAmount);
            // // '1'-多头开仓、'2'-多头平仓、'3'-空头平仓、'4'-空头开仓
            // tradeEntrustParam.setEntrustBs(EntrustBs.BUY.getCode());
            //
            // // 委托下多个订单
            // List<String> entrustIdList = Lists.newArrayList();
            // for (int i = 0; i < 10; i++) {
            //     ModelResult<CommonStringVo> tradeEntrustModelResult = quantOpenApiHandle.entrust(tradeEntrustParam);
            //     if (tradeEntrustModelResult.isSuccess() && tradeEntrustModelResult.getModel() != null) {
            //         entrustIdList.add(tradeEntrustModelResult.getModel().getData());
            //     }
            // }

            List<String> entrustIdList = Lists.newArrayList();
            entrustIdList.add("5630281");
            entrustIdList.add("5630280");
            entrustIdList.add("5630279");
            entrustIdList.add("5630278");
            entrustIdList.add("5630277");
            entrustIdList.add("5630276");
            entrustIdList.add("5630275");
            entrustIdList.add("5630274");
            entrustIdList.add("5630273");
            entrustIdList.add("5630272");
            // 根据交易类型批量撤单
            BatchCancelEntrustParam batchCancelEntrustParam = new BatchCancelEntrustParam();
            batchCancelEntrustParam.setExchangeType(ExchangeType.K.getCode());
            batchCancelEntrustParam.setEntrustId(entrustIdList);
            ModelResult<BatchCancelEntrustVo> modelResult = quantOpenApiHandle.batchCancelEntrust(batchCancelEntrustParam);

            // 根据订单号列表批量撤单
            // ModelResult<BatchCancelEntrustVo> modelResult = quantOpenApiHandle.tradeBatchCancelEntrust(exchangeType, entrustIdList);
        }

        /**
         * 条件单-下单/改单/撤单
         */
        public void entrustCond() {
            // 交易市场类型: 'K'港股、'P'美股、'v'深股通、't'沪股通
            String exchangeType = ExchangeType.K.getCode();
            String stockCode = "00700.HK";
            String entrustPrice = "295"; // 委托下单指定价格，当条件触发时的委托下单价格
            String entrustAmount = "1000";

            EntrustParam tradeEntrustParam = new EntrustParam();
            tradeEntrustParam.setStockCode(stockCode);
            tradeEntrustParam.setExchangeType(exchangeType);
            // 港股 ：'0'-竞价限价、'1'-竞价、'2'-增强限价盘、'3'-限价盘、'4'-特别限价盘、'6'-暗盘
            // 美股 ：'3'-限价盘、'5'-市价盘、'8'-冰山市价、'9'-冰山限价、'10'-隐藏市价、'11'-隐藏限价
            // A股  ：'3'-限价盘
            // 条件单：'31'-止盈限价单、'32'-止盈市价单(美股)、'33'-止损限价单、'34'-止损市价单(美股)、'35'-追踪止损限价单、'36'-追踪止损市价单(美股)
            EntrustType entrustType = EntrustType.COND_STOP_LOSS_POINT;
            tradeEntrustParam.setEntrustType(entrustType.getCode());
            tradeEntrustParam.setEntrustPrice(entrustPrice);
            tradeEntrustParam.setEntrustAmount(entrustAmount);
            // '1'-多头开仓、'2'-多头平仓、'3'-空头平仓、'4'-空头开仓
            tradeEntrustParam.setEntrustBs(EntrustBs.SELL.getCode());
            // 是否盘前盘后交易 
            // 3:只支持盘中 4:港股只支持暗盘 5:港股支持盘中及暗盘 6:美股只支持盘前盘后 7:美股支持盘中及盘前盘后 8:美股只支持盘前 9:美股只支持盘后
            tradeEntrustParam.setSessionType(SessionType.COND_ORDER_DEFAULT.getCode());
            // 有效天数
            tradeEntrustParam.setValidDays("10");
            // 示例是止损条件单，这里设置为止损价格
            tradeEntrustParam.setCondValue("300");
            // 跟踪条件单需要设置跟踪类型 1百分比、2价差
            // com.huasheng.quant.open.sdk.constant.TradeCondTrackType
            // tradeEntrustParam.setCondTrackType(1);

            // 委托下单
            ModelResult<CommonStringVo> tradeEntrustModelResult = quantOpenApiHandle.entrust(tradeEntrustParam);

            if (tradeEntrustModelResult.isSuccess() && tradeEntrustModelResult.getModel() != null) {
                CommonStringVo commonStringVo = tradeEntrustModelResult.getModel();
                String entrustId = commonStringVo.getData();

                String changeValidDays = "3"; // 修改条件单生效天数
                String changeCondValue = "280"; // 修改条件单止损价格

                ChangeEntrustParam changeEntrustParam = new ChangeEntrustParam();
                changeEntrustParam.setEntrustId(entrustId);
                changeEntrustParam.setStockCode(stockCode);
                changeEntrustParam.setExchangeType(exchangeType);
                // 条件单改单委托类型不允许修改，需传原委托类型
                changeEntrustParam.setEntrustType(entrustType.getCode());
                changeEntrustParam.setEntrustAmount(entrustAmount);
                changeEntrustParam.setEntrustPrice(entrustPrice);
                // 修改字段
                changeEntrustParam.setValidDays(changeValidDays);
                changeEntrustParam.setCondValue(changeCondValue);

                // 执行改单
                ModelResult<CommonStringVo> changeEntrustModelResult = quantOpenApiHandle.changeEntrust(changeEntrustParam);
                if (changeEntrustModelResult.isSuccess() && changeEntrustModelResult.getModel() != null) {

                    CancelEntrustParam cancelEntrustParam = new CancelEntrustParam();
                    cancelEntrustParam.setEntrustId(entrustId);
                    cancelEntrustParam.setStockCode(stockCode);
                    cancelEntrustParam.setExchangeType(exchangeType);
                    // 条件单撤单，委托类型必传，传原委托类型
                    cancelEntrustParam.setEntrustType(entrustType.getCode());

                    // 执行撤单
                    ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.cancelEntrust(cancelEntrustParam);
                }
            }
        }

        /**
         * 查询最大可用资产
         */
        public void queryMaxAvailableAsset() {
            MaxAvailableAssetParam maxAvailableAssetParam = new MaxAvailableAssetParam();
            maxAvailableAssetParam.setExchangeType(ExchangeType.K.getCode());
            maxAvailableAssetParam.setStockCode("00700.HK");
            maxAvailableAssetParam.setEntrustPrice("313");
            maxAvailableAssetParam.setEntrustType(EntrustType.AUCTION.getCode());
            maxAvailableAssetParam.setClientType(ClientType.INTERNET.getCode());

            quantOpenApiHandle.queryMaxAvailableAsset(maxAvailableAssetParam);
        }

        /**
         * 查询当日委托
         */
        public void queryRealEntrustList() {
            int queryCount = 20;
            RealEntrustPageQueryParam realEntrustPageQueryParam = new RealEntrustPageQueryParam();
            realEntrustPageQueryParam.setExchangeType(ExchangeType.K.getCode());
            realEntrustPageQueryParam.setQueryParamStr("0");
            realEntrustPageQueryParam.setQueryCount(queryCount);
            // 查询第一页数据
            ModelResult<OrderVo> modelResult = quantOpenApiHandle.queryRealEntrustList(realEntrustPageQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getData().size() > queryCount) {
                List<Order> data = modelResult.getModel().getData();
                realEntrustPageQueryParam.setQueryParamStr(data.get(data.size() - 1).getQueryParamStr());
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryRealEntrustList(realEntrustPageQueryParam);
            }
        }

        /**
         * 查询历史委托
         */
        public void queryHistoryEntrustList() {
            String queryParamStr = "0";
            int queryCount = 20;

            HistoryPageQueryParam historyPageQueryParam = new HistoryPageQueryParam();
            historyPageQueryParam.setExchangeType(ExchangeType.K.getCode());
            historyPageQueryParam.setQueryParamStr(queryParamStr);
            historyPageQueryParam.setQueryCount(queryCount);
            historyPageQueryParam.setStartDate("20221201");
            historyPageQueryParam.setEndDate("20221231");
            // 查询第一页数据
            ModelResult<OrderVo> modelResult = quantOpenApiHandle.queryHistoryEntrustList(historyPageQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getData().size() > queryCount) {
                List<Order> data = modelResult.getModel().getData();
                queryParamStr = data.get(data.size() - 1).getQueryParamStr();
                historyPageQueryParam.setQueryParamStr(queryParamStr);
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryHistoryEntrustList(historyPageQueryParam);
            }
        }

        /**
         * 查询当日成交
         */
        public void queryRealDeliverList() {
            // 交易市场类型: 'K'港股、'P'美股、'v'深股通、't'沪股通
            String exchangeType = ExchangeType.K.getCode();

            String queryParamStr = "0";
            int queryCount = 20;

            RealPageQueryParam realPageQueryParam = new RealPageQueryParam();
            realPageQueryParam.setExchangeType(exchangeType);
            realPageQueryParam.setQueryParamStr(queryParamStr);
            realPageQueryParam.setQueryCount(queryCount);

            // 查询第一页数据
            ModelResult<OrderVo> modelResult = quantOpenApiHandle.queryRealDeliverList(realPageQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getData().size() > queryCount) {
                List<Order> data = modelResult.getModel().getData();
                queryParamStr = data.get(data.size() - 1).getQueryParamStr();
                realPageQueryParam.setQueryParamStr(queryParamStr);
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryRealDeliverList(realPageQueryParam);
            }
        }

        /**
         * 查询历史成交
         */
        public void queryHistoryDeliverList() {
            // 交易市场类型: 'K'港股、'P'美股、'v'深股通、't'沪股通
            String exchangeType = ExchangeType.K.getCode();

            String queryParamStr = "0";
            int queryCount = 20;

            HistoryPageQueryParam historyPageQueryParam = new HistoryPageQueryParam();
            historyPageQueryParam.setExchangeType(exchangeType);
            historyPageQueryParam.setQueryParamStr(queryParamStr);
            historyPageQueryParam.setQueryCount(queryCount);
            historyPageQueryParam.setStartDate("20231101");
            historyPageQueryParam.setEndDate("20231102");

            // 查询第一页数据
            ModelResult<OrderVo> modelResult = quantOpenApiHandle.queryHistoryDeliverList(historyPageQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getData().size() > queryCount) {
                List<Order> data = modelResult.getModel().getData();
                queryParamStr = data.get(data.size() - 1).getQueryParamStr();
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryHistoryDeliverList(historyPageQueryParam);
            }
        }

        /**
         * 查询当日条件单
         */
        public void queryRealCondOrderList() {
            String exchangeType = ExchangeType.K.getCode();

            int pageNo = 1;
            int pageSize = 20;

            RealPageNoQueryParam realPageNoQueryParam = new RealPageNoQueryParam();
            realPageNoQueryParam.setExchangeType(exchangeType);
            realPageNoQueryParam.setPageNo(pageNo);
            realPageNoQueryParam.setPageSize(pageSize);

            // 查询第一页数据
            ModelResult<CondOrderVo> modelResult = quantOpenApiHandle.queryRealCondOrderList(realPageNoQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getTotalPages() > pageNo) {
                realPageNoQueryParam.setPageNo(pageNo + 1);
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryRealCondOrderList(realPageNoQueryParam);
            }
        }

        /**
         * 查询历史条件单
         */
        public void queryHistoryCondOrderList() {
            // 交易市场类型: 'K'港股、'P'美股、'v'深股通、't'沪股通
            String exchangeType = ExchangeType.K.getCode();

            String startTime = "2023-10-01 00:00:00";
            String endTime = "2023-11-04 00:00:00";

            int pageNo = 1;
            int pageSize = 20;

            HistoryPageNoQueryParam historyPageNoQueryParam = new HistoryPageNoQueryParam();
            historyPageNoQueryParam.setExchangeType(exchangeType);
            historyPageNoQueryParam.setStartTime(startTime);
            historyPageNoQueryParam.setEndTime(endTime);
            historyPageNoQueryParam.setPageNo(pageNo);
            historyPageNoQueryParam.setPageSize(pageSize);

            // 查询第一页数据
            ModelResult<CondOrderVo> modelResult = quantOpenApiHandle.queryHistoryCondOrderList(historyPageNoQueryParam);
            if (modelResult.isSuccess() && modelResult.getModel() != null && modelResult.getModel().getTotalPages() > pageNo) {
                historyPageNoQueryParam.setPageNo(pageNo + 1);
                sleep();
                // 查询第二页数据
                modelResult = quantOpenApiHandle.queryHistoryCondOrderList(historyPageNoQueryParam);
            }
        }

        /**
         * 查询是否支持盘前盘后交易
         */
        public void queryBeforeAndAfterSupport() {
            StockCodeParam stockCodeParam = new StockCodeParam();
            stockCodeParam.setExchangeType(ExchangeType.K.getCode());
            stockCodeParam.setStockCode("00700.HK");

            quantOpenApiHandle.queryBeforeAndAfterSupport(stockCodeParam);
        }

        /**
         * 查询股票沽空信息
         */
        public void queryStockShortInfo() {
            StockCodeParam stockCodeParam = new StockCodeParam();
            stockCodeParam.setExchangeType(ExchangeType.K.getCode());
            stockCodeParam.setStockCode("09988.HK");

            quantOpenApiHandle.queryStockShortInfo(stockCodeParam);
        }

        /**
         * 查询股票融资融券数据
         */
        public void queryMarginFullInfo() {
            DataTypeParam dataTypeParam = new DataTypeParam();
            dataTypeParam.setDataType(HqDataType.US_STOCK.getDataType().toString());
            dataTypeParam.setStockCode("AAPL");

            quantOpenApiHandle.queryMarginFullInfo(dataTypeParam);
        }

        /**
         * 订阅交易推送
         */
        public void subscribe() {
            quantOpenApiHandle.subscribe();
        }

        /**
         * 取消订阅交易推送
         */
        public void unsubscribe() {
            quantOpenApiHandle.unsubscribe();
        }


        /**
         * 期货-下单/改单/撤单
         */
        public void futuresEntrust() {
            String code = "HHI2311.HK";

            FuturesEntrustParam futuresEntrustParam = new FuturesEntrustParam();
            futuresEntrustParam.setStockCode(code);
            futuresEntrustParam.setEntrustType(FuturesEntrustType.LIMIT.getCode());
            futuresEntrustParam.setEntrustPrice("5977");
            futuresEntrustParam.setEntrustAmount("1");
            futuresEntrustParam.setEntrustBs(FuturesEntrustBs.BUY.getCode());
            futuresEntrustParam.setValidTimeType(FuturesValidTimeType.DEAL_AND_CANCEL.getCode());
            futuresEntrustParam.setValidTime("20231103");
            // 执行订单委托
            ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.futuresEntrust(futuresEntrustParam);
            sleep();

            if (modelResult.isSuccess() && modelResult.getModel() != null) {
                // 【期货改单接口】
                String entrustId = modelResult.getModel().getData();
                FuturesChangeEntrustParam modifyFuturesEntrustParam = new FuturesChangeEntrustParam();
                modifyFuturesEntrustParam.setEntrustId(entrustId);
                modifyFuturesEntrustParam.setStockCode(code);
                modifyFuturesEntrustParam.setEntrustPrice("6.1");
                modifyFuturesEntrustParam.setEntrustAmount("100");
                modifyFuturesEntrustParam.setEntrustBs(FuturesEntrustBs.BUY.getCode());
                modifyFuturesEntrustParam.setValidTimeType(FuturesValidTimeType.VALID_TODAY.getCode());
                modifyFuturesEntrustParam.setValidTime("20221231");
                // 执行改单
                ModelResult<CommonStringVo> modelResult1 = quantOpenApiHandle.futuresModifyEntrust(modifyFuturesEntrustParam);
                sleep();

                // 【期货撤单接口】
                // 执行撤单
                if (modelResult1.isSuccess() && modelResult1.getModel() != null) {
                    entrustId = modelResult1.getModel().getData();

                    FuturesEntrustIdParam futuresEntrustIdParam = new FuturesEntrustIdParam();
                    futuresEntrustIdParam.setStockCode(code);
                    futuresEntrustIdParam.setEntrustId(entrustId);

                    quantOpenApiHandle.futuresCancelEntrust(futuresEntrustIdParam);
                }
            }
        }

        public void futuresModifyEntrust() {
            String entrustId = "OA202311020000000010";
            String code = "HHI2311.HK";

            FuturesChangeEntrustParam futuresChangeEntrustParam = new FuturesChangeEntrustParam();
            futuresChangeEntrustParam.setEntrustId(entrustId);
            futuresChangeEntrustParam.setStockCode(code);
            futuresChangeEntrustParam.setEntrustPrice("5966");
            futuresChangeEntrustParam.setEntrustAmount("2");
            futuresChangeEntrustParam.setEntrustBs(FuturesEntrustBs.BUY.getCode());
            futuresChangeEntrustParam.setValidTimeType(FuturesValidTimeType.DEAL_AND_CANCEL.getCode());
            futuresChangeEntrustParam.setValidTime("20231103");

            // 执行改单
            ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.futuresModifyEntrust(futuresChangeEntrustParam);
        }

        public void futuresCancelEntrust() {
            String entrustId = "OA202311020000000010";
            String code = "HHI2311.HK";

            FuturesEntrustIdParam futuresEntrustIdParam = new FuturesEntrustIdParam();
            futuresEntrustIdParam.setStockCode(code);
            futuresEntrustIdParam.setEntrustId(entrustId);

            quantOpenApiHandle.futuresCancelEntrust(futuresEntrustIdParam);
        }

        /**
         * 期货-查询今日委托
         */
        public void futuresQueryRealEntrustList() {
            quantOpenApiHandle.futuresQueryRealEntrustList();
        }

        /**
         * 期货-查询历史委托
         */
        public void futuresQueryHistoryEntrustList() {
            FuturesPageQueryParam futuresPageQueryParam = new FuturesPageQueryParam();
            futuresPageQueryParam.setPageNo(1);
            futuresPageQueryParam.setPageSize(5);
            futuresPageQueryParam.setStartDate("20231102");
            futuresPageQueryParam.setEndDate("20231102");
            quantOpenApiHandle.futuresQueryHistoryEntrustList(futuresPageQueryParam);
        }

        /**
         * 期货-查询今日成交
         */
        public void futuresQueryRealDeliverList() {
            quantOpenApiHandle.futuresQueryRealDeliverList();
        }

        /**
         * 期货-查询历史成交
         */
        public void futuresQueryHistoryDeliverList() {
            FuturesPageQueryParam futuresPageQueryParam = new FuturesPageQueryParam();
            futuresPageQueryParam.setPageNo(1);
            futuresPageQueryParam.setPageSize(5);
            futuresPageQueryParam.setStartDate("20231102");
            futuresPageQueryParam.setEndDate("20231103");
            quantOpenApiHandle.futuresQueryHistoryDeliverList(futuresPageQueryParam);
        }

        /**
         * 策略单-下单
         */
        public void algoEntrust() {
            String stockCode = "00700.HK";
            String exchangeType = ExchangeType.K.getCode();

            String entrustPrice = "286";
            String entrustAmount = "300";

            AlgoEntrustParam algoEntrustParam = new AlgoEntrustParam();
            algoEntrustParam.setStockCode(stockCode);
            algoEntrustParam.setExchangeType(exchangeType);
            algoEntrustParam.setEntrustType(AlgoEntrustType.LIMIT.getVal()); // 委托类型 限价单、市价单
            algoEntrustParam.setEntrustPrice(entrustPrice);
            algoEntrustParam.setEntrustAmount(entrustAmount);
            algoEntrustParam.setEntrustBs(EntrustBs.BUY.getCode());
            algoEntrustParam.setTargetStrategy(AlgoStrategyType.VWAP.getVal()); // 策略类型
            algoEntrustParam.setSessionType(SessionType.GENERAL_ORDER_YES.getCode()); // 盘前盘后交易

            AlgoStrategy algoStrategyParam = new AlgoStrategy();
            algoStrategyParam.setOrigStartTime("103015"); // 开始时间
            algoStrategyParam.setOrigEndTime("103415"); // 结束时间
            algoStrategyParam.setMaxVolume("100"); // 每笔子单最大的委托数量
            algoStrategyParam.setSensitivity(AlgoStrategySensitivityType.AGGRESSIVE.getVal()); // 订单的执行效率

            algoEntrustParam.setStrategyParam(algoStrategyParam);

            ModelResult<CommonStringVo> modelResult = quantOpenApiHandle.algoEntrust(algoEntrustParam);
        }

        /**
         * 策略单-撤销母单
         */
        public void algoCancelOrder() {
            AlgoOrderIdParam algoOrderIdParam = new AlgoOrderIdParam();
            algoOrderIdParam.setExchangeType(ExchangeType.K.getCode());
            algoOrderIdParam.setOrderId("11741");

            quantOpenApiHandle.algoCancelOrder(algoOrderIdParam);
        }

        /**
         * 策略单-撤销子单
         */
        public void algoCancelSubOrder() {
            AlgoCancelEntrustParam algoCancelEntrustParam = new AlgoCancelEntrustParam();
            algoCancelEntrustParam.setExchangeType(ExchangeType.K.getCode());
            algoCancelEntrustParam.setOrderId("11735");
            algoCancelEntrustParam.setEntrustId("5631699");

            quantOpenApiHandle.algoCancelSubOrder(algoCancelEntrustParam);
        }

        /**
         * 策略单-改单
         */
        public void algoModifyOrder() {
            String exchangeType = ExchangeType.K.getCode();
            String stockCode = "00700.HK";
            String orderId = "11742";

            AlgoChangeEntrustParam algoChangeEntrustParam = new AlgoChangeEntrustParam();
            algoChangeEntrustParam.setOrderId(orderId);
            algoChangeEntrustParam.setStockCode(stockCode);
            algoChangeEntrustParam.setExchangeType(exchangeType);
            algoChangeEntrustParam.setEntrustPrice("285");
            algoChangeEntrustParam.setEntrustAmount("200");

            AlgoStrategy algoStrategyParam = new AlgoStrategy();
            // algoStrategyParam.setOrigStartTime("165530"); // 开始时间
            // algoStrategyParam.setOrigEndTime("170530"); // 结束时间
            algoStrategyParam.setMaxVolume("100"); // 每笔子单最大的委托数量
            algoStrategyParam.setSensitivity(AlgoStrategySensitivityType.NEUTRAL.getVal()); // 订单的执行效率

            algoChangeEntrustParam.setStrategyParam(algoStrategyParam);
            quantOpenApiHandle.algoModifyOrder(algoChangeEntrustParam);
        }

        /**
         * 策略单-操作
         */
        public void algoOrderAction() {
            AlgoActionParam algoActionParam = new AlgoActionParam();
            algoActionParam.setExchangeType(ExchangeType.K.getCode());
            algoActionParam.setOrderId("11741");
            algoActionParam.setAction(AlgoActionType.SUSPEND.getVal());
            algoActionParam.setTargetStrategy(AlgoStrategyType.TWAP.getVal());

            quantOpenApiHandle.algoOrderAction(algoActionParam);
        }

        /**
         * 策略单-查询母单列表
         */
        public void algoQueryOrderList() {
            AlgoQueryPageParam algoQueryPageParam = new AlgoQueryPageParam();
            algoQueryPageParam.setStartDate("20231103");
            algoQueryPageParam.setEndDate("20231103");
            algoQueryPageParam.setPageNo(1);
            algoQueryPageParam.setPageSize(10);

            quantOpenApiHandle.algoQueryOrderList(algoQueryPageParam);
        }

        /**
         * 策略单-查询子单
         */
        public void algoQuerySubOrderIdList() {
            AlgoQueryEntrustIdParam algoQueryEntrustIdParam = new AlgoQueryEntrustIdParam();
            algoQueryEntrustIdParam.setExchangeType(ExchangeType.K.getCode());
            algoQueryEntrustIdParam.setTradeDate("20231102");
            algoQueryEntrustIdParam.setOrderId("11735");

            quantOpenApiHandle.algoQuerySubOrderIdList(algoQueryEntrustIdParam);
        }

        /**
         * 查询基础报价
         */
        public void queryHqBasicQot() {
            List<Security> list = new ArrayList<>();

            list.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            HqBasicQotParam hqBasicQotParam = new HqBasicQotParam(list, HqNeedDelayFlag.NO.getCode(), MktTmType.IN_MARKET.getCode());

            ModelResult<BasicQotVo> modelResult = quantOpenApiHandle.queryHqBasicQot(hqBasicQotParam);
        }

        /**
         * 查询买卖经纪摆盘
         */
        public void queryHqBroker() {
            Security security = new Security(HqDataType.HK_STOCK.getDataType(), "09988.HK");

            ModelResult<BrokerVo> modelResult = quantOpenApiHandle.queryHqBroker(new SecurityParam(security));
        }

        /**
         * 查询买卖档
         */
        public void queryHqOrderBook() {
            Security security = new Security(HqDataType.HK_STOCK.getDataType(), "09988.HK");
            OrderBookParam orderBookParam = new OrderBookParam(security, MktTmType.IN_MARKET.getCode(), null);

            ModelResult<OrderBookVo> modelResult = quantOpenApiHandle.queryHqOrderBook(orderBookParam);
        }

        /**
         * 查询逐笔
         */
        public void queryHqTicker() {
            Security security = new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK");

            ModelResult<TickerVo> modelResult = quantOpenApiHandle.queryHqTicker(new TickerParam(security, 50, MktTmType.IN_MARKET.getCode()));
        }

        /**
         * 查询K线
         */
        public void queryHqKLine() {
            Security security = new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK");

            ModelResult<KLineVo> modelResult = quantOpenApiHandle.queryHqKLine(new KLineParam(security, 20230901L, Direction.RIGHT.getCode(), ExRightFlag.NO_REINSTATEMENT.getCode(), CycType.DAY_LINE.getCode(), 10));
        }

        /**
         * 查询分时
         */
        public void queryHqTimeShare() {
            Security security = new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK");

            ModelResult<TimeShareVo> modelResult = quantOpenApiHandle.queryHqTimeShare(new TimeShareParam(security, MktTmType.IN_MARKET.getCode()));
        }

        /**
         * 查询期权链到期日列表
         */
        public void queryHqUsOptionChainExpireDate() {
            String code = "BABA";

            ModelResult<OptionChainExpireDateVo> modelResult = quantOpenApiHandle.queryHqUsOptionChainExpireDate(new OptionParam(code));
        }

        /**
         * 查询期权链Code列表
         */
        public void queryHqUsOptionChainCode() {
            String code = "BABA";

            ModelResult<OptionChainCodeVo> modelResult = quantOpenApiHandle.queryHqUsOptionChainCode(new OptionChainParam(code, "20231220", OptionPriceFlag.PRICE_IN.getVal(), OptionType.CALL.getCode()));
        }

        /**
         * 订阅行情基础报价推送消息
         */
        public void hqSubscribeBasicQot() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "09988.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "TSLA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.BASIC_QOT.getTopicId(), securityList));
        }

        /**
         * 订阅行情买卖经纪推送消息
         */
        public void hqSubscribeBroker() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.BROKER.getTopicId(), securityList));
        }

        /**
         * 订阅行情买卖档推送消息
         */
        public void hqSubscribeOrderBook() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "09988.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.ORDER_BOOK.getTopicId(), securityList));
        }

        /**
         * 订阅行情逐笔推送消息
         */
        public void hqSubscribeTicker() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.TICKER.getTopicId(), securityList));
        }

        /**
         * 订阅行情深度摆盘-ARCABOOK买卖档推送消息
         */
        public void hqSubscribeArcaBook() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.ORDER_BOOK_ARCABOOK.getTopicId(), securityList));
        }

        /**
         * 订阅行情深度摆盘-TOTALVIEW买卖档推送消息
         */
        public void hqSubscribeTotalView() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqSubscribe(new HqSubscribeParam(HqSubscribeTopic.ORDER_BOOK_TOTALVIEW.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情基础报价推送消息
         */
        public void hqUnsubscribeBasicQot() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            // securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.BASIC_QOT.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情买卖经纪推送消息
         */
        public void hqUnsubscribeBroker() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.BROKER.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情买卖档推送消息
         */
        public void hqUnsubscribeOrderBook() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.ORDER_BOOK.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情逐笔推送消息
         */
        public void hqUnsubscribeTicker() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.TICKER.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情深度摆盘-ARCABOOK买卖档推送消息
         */
        public void hqUnsubscribeArcaBook() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.ORDER_BOOK_ARCABOOK.getTopicId(), securityList));
        }

        /**
         * 取消订阅行情深度摆盘-TOTALVIEW买卖档推送消息
         */
        public void hqUnsubscribeTotalView() {
            List<Security> securityList = new ArrayList<>();
            securityList.add(new Security(HqDataType.HK_STOCK.getDataType(), "00700.HK"));
            securityList.add(new Security(HqDataType.US_STOCK.getDataType(), "BABA"));

            quantOpenApiHandle.hqUnsubscribe(new HqUnsubscribeParam(HqSubscribeTopic.ORDER_BOOK_TOTALVIEW.getTopicId(), securityList));
        }

        public void stop() {
            try {
                // 断开与Server端得TCP连接【！！！该方法请在程序停止时执行！！！】
                quantOpenApiHandle.disconnectAndStop();
                logger.info("**************** Process exit. ****************");
            } catch (Exception e) {
            }
        }

        private void addShutdownHook() {
            Runtime.getRuntime().addShutdownHook(new Thread(() -> {
                try {
                    // 断开与Server端得TCP连接【！！！该方法请在程序停止时执行！！！】
                    quantOpenApiHandle.disconnectAndStop();
                    logger.info("**************** Shutdown hook exit. ****************");
                } catch (Exception e) {
                }
            }, "StartMain-shutdown-hook"));
        }

        public void sleep() {
            try {
                TimeUnit.MILLISECONDS.sleep(sleepTime);
            } catch (InterruptedException e) {
            }
        }
    }

}
